@@ -13,7 +13,7 @@ use swc_ecmascript::ast::{
   MemberExpr, MethodKind, NamedExport, Param, Pat, Prop, PropName, SetterProp,
   TsEntityName, TsEnumDecl, TsExprWithTypeArgs, TsInterfaceDecl, TsModuleDecl,
   TsNamespaceDecl, TsPropertySignature, TsTypeAliasDecl, TsTypeQueryExpr,
-  TsTypeRef, VarDecl, VarDeclOrPat, VarDeclarator,
+  TsTypeRef, VarDecl, VarDeclarator,
 };
 use swc_ecmascript::utils::ident::IdentLike;
 use swc_ecmascript::utils::{find_ids, Id};
@@ -81,6 +81,8 @@ impl LintRule for NoUnusedVars {
   }
 }
 
+// TODO(@magurotuna): use MediaType instead
+// https://github.com/denoland/deno/blob/76e2edc7e1868d7768e259aacbb9a991e1afc462/cli/media_type.rs#L15-L26
 fn is_jsx_file(filename: &str) -> bool {
   filename.ends_with(".jsx") || filename.ends_with(".tsx")
 }
@@ -291,25 +293,6 @@ impl Visit for Collector {
     self.used_vars.insert(export.orig.to_id());
   }
 
-  /// Handle for-in/of loops
-  fn visit_var_decl_or_pat(&mut self, node: &VarDeclOrPat, _: &dyn Node) {
-    // We need this because find_ids searches ids only in the pattern.
-    node.visit_children_with(self);
-
-    match node {
-      VarDeclOrPat::VarDecl(v) => {
-        // This is declaration, but cannot be removed.
-        let ids = find_ids(v);
-        self.used_vars.extend(ids);
-      }
-      VarDeclOrPat::Pat(p) => {
-        // This is assignment, but cannot be removed
-        let ids = find_ids(p);
-        self.used_vars.extend(ids);
-      }
-    }
-  }
-
   fn visit_fn_decl(&mut self, decl: &FnDecl, _: &dyn Node) {
     let id = decl.ident.to_id();
     self.with_cur_defining(iter::once(id), |a| {
@@ -736,16 +719,6 @@ mod tests {
       "var _a",
       "function foo(_a) { } foo();",
       "function foo(a, _b) { return a; } foo();",
-      "(function(obj) { var name; for ( name in obj ) return; })({});",
-      "(function(obj) { var name; for ( name in obj ) { return; } })({});",
-      "(function(obj) { for ( var name in obj ) { return true } })({})",
-      "(function(obj) { for ( var name in obj ) return true })({})",
-      "(function(obj) { let name; for ( name in obj ) return; })({});",
-      "(function(obj) { let name; for ( name in obj ) { return; } })({});",
-      "(function(obj) { for ( let name in obj ) { return true } })({})",
-      "(function(obj) { for ( let name in obj ) return true })({})",
-      "(function(obj) { for ( const name in obj ) { return true } })({})",
-      "(function(obj) { for ( const name in obj ) return true })({})",
       "try{}catch(err){console.error(err);}",
       "try{}catch(_ignoreErr){}",
       "var a = 0, b; b = a = a + 1; foo(b);",
@@ -1327,6 +1300,29 @@ export class Foo {
   constructor(xy: number | string) {
     console.log(xy);
   }
+}
+      "#,
+
+      // https://github.com/denoland/deno_lint/issues/705
+      r#"
+type Foo = string[];
+function _bar(...Foo: Foo): void {
+  console.log(Foo);
+}
+      "#,
+      r#"
+import type { Foo } from "./foo.ts";
+function _bar(...Foo: Foo) {
+  console.log(Foo);
+}
+      "#,
+      r#"
+import type { Filters } from "./types.ts";
+export function audioFilters(...Filters: Filters[]): void {
+  for (const filter of Filters) {
+    console.log(filter);
+    return;
+  }
 }
       "#,
     };
@@ -1334,15 +1330,12 @@ export class Foo {
     // JSX or TSX
     assert_lint_ok! {
       NoUnusedVars,
-      {
-        src: "
-    import { TypeA } from './interface';
-    export const a = <GenericComponent<TypeA> />;
-        ",
-        filename: "foo.tsx",
-      },
-      {
-        src: r#"
+      filename: "foo.tsx",
+      r#"
+import { TypeA } from './interface';
+export const a = <GenericComponent<TypeA> />;
+      "#,
+      r#"
 const text = 'text';
 export function Foo() {
   return (
@@ -1351,50 +1344,33 @@ export function Foo() {
     </div>
   );
 }
-        "#,
-        filename: "foo.tsx",
-      },
-      {
-        src: r#"
+      "#,
+      r#"
 function Root() { return null; }
 function Child() { return null; }
 export default <Root><Child>Hello World!</Child></Root>;
-        "#,
-        filename: "foo.tsx",
-      },
+      "#,
 
       // https://github.com/denoland/deno_lint/issues/663
-      {
-        src: r#"
+      r#"
 import React from "./dummy.ts";
 export default <div />;
-        "#,
-        filename: "foo.tsx",
-      },
-      {
-        src: r#"
+      "#,
+      r#"
 import React from "./dummy.ts";
 function Component() { return null; }
 export default <Component />;
-        "#,
-        filename: "foo.tsx",
-      },
-      {
-        src: r#"
+      "#,
+      r#"
 import React from "./dummy.ts";
 const Component = () => { return null; }
 export default <Component />;
-        "#,
-        filename: "foo.tsx",
-      },
-      {
-        src: r#"
+      "#,
+      r#"
 import React from "./dummy.ts";
 class Component extends React.Component { render() { return null; } }
 export default <Component />;
-        "#,
-        filename: "foo.tsx",
-      },
+      "#,
     };
   }
 
@@ -2067,6 +2043,88 @@ foo(a);
           hint: variant!(NoUnusedVarsHint, AddPrefix, "fn"),
         }
       ],
+
+      // https://github.com/denoland/deno_lint/issues/697
+      "for (const x of xs) {}": [
+        {
+          col: 11,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x"),
+        }
+      ],
+      "let x; for (x of xs) {}": [
+        {
+          col: 4,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x"),
+        }
+      ],
+      "for await (const x of xs) {}": [
+        {
+          col: 17,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x"),
+        }
+      ],
+      "let x; for await (x of xs) {}": [
+        {
+          col: 4,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x"),
+        }
+      ],
+      "for (const x in xs) {}": [
+        {
+          col: 11,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x"),
+        }
+      ],
+      "let x; for (x in xs) {}": [
+        {
+          col: 4,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x"),
+        }
+      ],
+      "for (const [x1, x2] of xs) {}": [
+        {
+          col: 12,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x1"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x1"),
+        },
+        {
+          col: 16,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x2"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x2"),
+        }
+      ],
+      "let x1, x2; for ([x1, x2] of xs) { console.log(x1); }": [
+        {
+          col: 8,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x2"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x2"),
+        },
+      ],
+      "for (const { x1, x2 } of xs) {}": [
+        {
+          col: 13,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x1"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x1"),
+        },
+        {
+          col: 17,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x2"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x2"),
+        }
+      ],
+      "let x1, x2; for ({ x1, x2 } of xs) { console.log(x2); }": [
+        {
+          col: 4,
+          message: variant!(NoUnusedVarsMessage, NeverUsed, "x1"),
+          hint: variant!(NoUnusedVarsHint, AddPrefix, "x1"),
+        },
+      ],
     };
   }
 
